local addonName, addonData = ...;
local events = {};
-- Frames
local dungeonFrame = CreateFrame("Frame");

-- Variables
local charNameWithRealm;

local goldLootedTotal = 0;
local bonusLootedTotal = 0;
local itemsLootedVendor = 0;
local lastDungeonName = "";
local lastDungeonDifficulty = "";
local enterTime = 0;

local itemsLooted = {};

local function TableToCoinString(coinTable)
	local coinString = 0;
	
	local count = 0;
	local lootCount = #coinTable;
	for _,v in pairs(coinTable) do
		if (lootCount == 3) then
			if (count == 0) then
				coinString = v * 10000;
			elseif (count == 1) then
				coinString = coinString + (v * 100);
			else
				coinString = coinString + v;
			end
		elseif (lootCount == 2) then
			if (count == 0) then
				coinString = coinString + (v * 100);
			else
				coinString = coinString + v;
			end
		else
			coinString = v;
		end
		
		count = count + 1;
	end
	
	return coinString;
end

local function GetTotalResult()
	local result = {
		goldLooted = goldLootedTotal,
		bonusLooted = bonusLootedTotal
	};
	if (bonusLootedTotal > 0) then
		result.text = format("Looted since entering: %s (bonus: %s)", GetCoinTextureString(goldLootedTotal), GetCoinTextureString(bonusLootedTotal));
	else
		result.text = format("Looted since entering: %s", GetCoinTextureString(goldLootedTotal));
	end
	
	return result;
end

local function ResetLoot()
	goldLootedTotal = 0;
	bonusLootedTotal = 0;
	itemsLootedVendor = 0;
end

local function ResetDungeonTracking()
	enterTime = 0;
	lastDungeonName = "";
	lastDungeonDifficulty = "";
	
	itemsLooted = {};
end

function DLU.ResetDungeonReport()
	ResetLoot();
	ResetDungeonTracking();
	
	-- Reset UI
	DLUDungeonReportUI.dungeonName:SetText(addonData.L.DungeonName .. addonData.L.Unknown);
	DLUDungeonReportUI.rawGold:SetText(addonData.L.RawGold .. GetCoinTextureString(0));
	DLUDungeonReportUI.rawGoldVendor:SetText(addonData.L.RawVendorGold .. GetCoinTextureString(0));
	DLUDungeonReportUI.timeElapsed:SetText(addonData.L.RunTime .. addonData.L.CalculatedOnExit);
	
	if (DLUCharacterSettings.farmingModeResetNotification.value) then
		DLU.createMessage("Dungeon report has been reset");
	end
end

function events:ZONE_CHANGED_NEW_AREA()
	if (DLUCharacterSettings.farmingMode.value) then
		local inInstance, instanceType = IsInInstance();
		local instanceName, instanceTypeInfo, _, difficultyName = GetInstanceInfo();
		
		--print("New zone!", inInstance, instanceType, instanceName, instanceTypeInfo);
		
		if ((inInstance and instanceType == "party" or instanceType == "raid") or (instanceName and instanceTypeInfo == "party" or instanceType == "raid")) then
			if (DLUCharacterSettings.farmingModeDungeonOpenWindow.value) then
				DLUDungeonReportUI:Show();
			end
			
			--local instanceName, _, _, difficultyName = GetInstanceInfo();
			enterTime = GetTime();
			
			if (instanceName ~= lastDungeonName) then
				lastDungeonName = instanceName;
			end
			
			if (lastDungeonDifficulty ~= difficultyName) then
				lastDungeonDifficulty = difficultyName;
			end
			
			DLUDungeonReportUI.dungeonName:SetText(addonData.L.DungeonName .. format("%s (%s)", instanceName, difficultyName));
		else
			if (enterTime ~= 0 and lastDungeonName ~= "" and lastDungeonDifficulty ~= "") then
				local result = GetTotalResult();
				
				local timeElapsed = GetTime() - enterTime;
				
				local totalLooted = result.goldLooted;
				if (result.bonusLooted > 0) then
					totalLooted = totalLooted + result.bonusLooted;
				end
				
				local goldPerHour = (totalLooted / (math.floor(timeElapsed*1000)/1000)) * 60;
				
				if (result.bonusLooted > 0) then
					DLUDungeonReportUI.rawGold:SetText(addonData.L.RawGold .. format(addonData.L.RawGoldBonus, GetCoinTextureString(totalLooted), GetCoinTextureString(result.goldLooted), GetCoinTextureString(result.bonusLooted)));
				else
					DLUDungeonReportUI.rawGold:SetText(addonData.L.RawGold .. GetCoinTextureString(result.goldLooted));
				end
				
				--print(format("Raw gold from selling items to vendor %s", GetCoinTextureString(itemsLootedVendor)));
				
				local vendorAndLooted = itemsLootedVendor + totalLooted;
				
				--print(format("Raw gold from selling items to vendor and looting %s", GetCoinTextureString(vendorAndLooted)));
				
				--DLUDungeonReportUI.rawGoldHour:SetText(addonData.L.RawGoldHour .. GetCoinTextureString(goldPerHour));
				DLUDungeonReportUI.timeElapsed:SetText(addonData.L.RunTime .. SecondsToTime(timeElapsed));
				
				if (DLUCharacterSettings.farmingModeDungeonExitOpenWindow.value) then
					DLUDungeonReportUI:Show();
				end
				
				if (DLUCharacterSettings.farmingModeResetDungeonOnExit.value and lastDungeonName ~= "") then
					ResetInstances();
				end
				
				ResetLoot();
				
				-- Reset dungeon tracking
				ResetDungeonTracking();
				
				if (DLUCharacterSettings.farmingModeResetNotification.value) then
					DLU.createMessage("Dungeon report has been reset");
				end
			end
		end
	end
end

function events:CHAT_MSG_MONEY(message)
	local inInstance, instanceType = IsInInstance();
	if (inInstance and instanceType == "party" or instanceType == "raid" and DLUCharacterSettings.farmingMode.value) then
		local looted, bonus = message:match( "(.+)%((.+)%)" );
		
		if (looted == nil) then
			looted = message;
		end
		
		local goldLooted = {};
		for v in looted:lower():gmatch("%d+") do
			table.insert(goldLooted, v);
		end
		
		local goldBonusLooted = {};
		if (bonus) then
			for v in bonus:lower():gmatch("%d+") do
				table.insert(goldBonusLooted, v);
			end
		end
		
		local looted = TableToCoinString(goldLooted);
		local bonusLooted = TableToCoinString(goldBonusLooted);
		
		goldLootedTotal = goldLootedTotal + looted;
		bonusLootedTotal = bonusLootedTotal + bonusLooted;
		
		DLUDungeonReportUI.rawGold:SetText(addonData.L.RawGold .. GetCoinTextureString(goldLootedTotal));
	end
end

function events:CHAT_MSG_LOOT(msg, playerName)
	if (playerName == charNameWithRealm and DLUCharacterSettings.farmingMode.value) then
		local itemId = msg:match("item:(%d+):");
		local quantity = msg:match("x(%d+)") or 1;
		
		if (itemsLooted[itemId]) then
			local existing = itemsLooted[itemId];
			existing.quantity = existing.quantity + quantity;
		else
			itemsLooted[itemId] = {
				id = itemId,
				quantity = quantity
			};
		end
		
		local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice = GetItemInfo(itemId);
		local vendorValue = itemSellPrice * quantity;
		
		itemsLootedVendor = itemsLootedVendor + vendorValue;
		
		DLUDungeonReportUI.rawGoldVendor:SetText(addonData.L.RawVendorGold .. GetCoinTextureString(itemsLootedVendor));
	end
end

-- Initialization
dungeonFrame:RegisterEvent("PLAYER_LOGIN");
dungeonFrame:SetScript("OnEvent", function()
	local fullName, realm = UnitFullName("player");
	charNameWithRealm = fullName .. "-" .. realm;
		
	dungeonFrame:SetScript("OnEvent", function(self, event, ...)
		events[event](self, ...);
	end);
	for k, v in pairs(events) do
		dungeonFrame:RegisterEvent(k);
	end
	
	dungeonFrame:UnregisterEvent("PLAYER_LOGIN")
end);